
# $Id: LnInput.pm 166 2006-04-21 06:42:08Z manu $

package PARALOOP::PLUGIN::LnInput;

=pod

=head1 Copyright Notice

 Copyright I.N.R.A. - C.N.R.S. 

 emmanuel.courcelle@toulouse.inra.fr
 jerome.gouzy@toulouse.inra.fr

 This software is a computer program whose purpose is to help you run 
 your scripts on parallel computers.

 This software is governed by the CeCILL  license under French law and
 abiding by the rules of distribution of free software.  You can  use, 
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 http://www.cecill.info" 

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability. 

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or 
 data to be ensured and,  more generally, to use and operate it in the 
 same conditions as regards security. 

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

LnInput - Plugins reading text files (1 record = 1 line) should derive from LnInput

=head1 Derives from...

PARALOOP::PLUGIN::Plugin

=head1 PUBLIC METHODS

 New 
     my $obj = LnInput->New(-inputfile=>$file,-log=>$log);

 NextRecord
     $obj->NextRecord();

 TellLength
     my $nb_of_records = $obj->TellLength()

 Seek
    my $obj->Seek(10);

 SkipRecords
     $obj->SkipRecords(10)

=pod

=head1 METHODS AND SUBROUTINES - DETAILED DESCRIPTION

=cut

use strict;
use warnings;
use Carp;
use IO::File;

use Logger;
use PARALOOP::PLUGIN::Plugin;

our @ISA = qw( PARALOOP::PLUGIN::Plugin );

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 166 $ =~ /\d+/g); $r[0]};
}

=pod

=head2 procedure _Init

 Usage:     CALLED BY UNIVERSAL::New
            my $obj = LnInput->New(-inputfile=>$file,-log=>$log);
 Access:    private (use New instead)
 procedure: Init the object, opening the input file
            update private member __records_counter
 Args:      inputfile: the file name
            log: a Logger object, already initialized

=cut

sub _Init
{
    my $self = shift;
    my %args = @_;
    my $inputfile=$args{'-inputfile'};
    delete $args{'-inputfile'};
    $self->SUPER::_Init(%args);     # Init the superclass

    $self->{__inputfile} = $inputfile;
    $self->{__log}       = $args{'-log'};
    my $inputfd = new IO::File($inputfile) or croak "ERROR - Cannot open file $inputfile";
    $self->{__inputfd}         = $inputfd;
    $self->{__records_counter} = 0;
    return;
}

=pod

=head2 function NextRecord

 Usage:    my $line = $obj->NextRecord()
 function: read and return next record (a string)
           update private member __records_counter
 Return:   a string
 Args:     none

=cut

sub NextRecord
{
    my $self    = shift;
    my $inputfd = $self->{"__inputfd"};
    $self->{"__records_counter"}++;
    my $line = <$inputfd>;
    chomp($line) if (defined $line);
    return $line;
}

=pod

=head2 function Tell

 Usage:    my $cnt = $obj->Tell()
 function: return records_counter private field
 Return:   The counter of records (first=0)
 Args:     none

=cut

sub Tell
{
    my $self = shift;
    return $self->{"__records_counter"};
}

=pod

=head2 procedure Seek

 Usage:    $obj->Seek($n)
 procedure:Position the pointer to $n (absolute, 1st record is 0)
 Args:     The record to seek to (MUST BE > 0)

=cut

sub Seek 
{
    my $self = shift;
    my $n = shift;
    my $inputfd = $self->{"__inputfd"};
    $inputfd->seek(0,0);               # Rewind file
    $self->{"__records_counter"} = 0;
    if ($n > 0)
    {
        $self->SkipRecords($n);
    };
}

=pod

=head2 function TellLength

 Usage:    my $cnt = $obj->TellLength()
 function: Return the number of lines found in the input file
 Return:   The total number of records
 Args:     none

=cut

sub TellLength
{
    my $self      = shift;
    my $inputfile = $self->{__inputfile};
    my $rvl       = `wc -l $inputfile`;
    chomp($rvl);
    my @rvl = ($rvl =~ /(\d+)/);
    return $rvl[0];
}

=head2 procedure SkipRecords

 Usage:    $obj->SkipRecords($n)
 procedure:Position the pointer, skipping $n records
 Args:     The number of records to skip (MUST BE > 0)

=cut

sub SkipRecords
{
    my $self    = shift;
    my $n       = shift;
    my $inputfd = $self->{"__inputfd"};
    croak("ERROR - CANNOT skip_record with parameter <= 0 !!!") if ($n <= 0);
    for (my $i = 0; $i < $n; $i++)
    {
        <$inputfd>;
    }
    $self->{"__records_counter"} += $n;
}

1;

